@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 2009-2010 Erik de Castro Lopo <erikd@mega-nerd.com>
+** Copyright (C) 2009-2015 Erik de Castro Lopo <erikd@mega-nerd.com>
 **
 ** All rights reserved.
 **
@@ -60,46 +60,54 @@ typedef struct
 } STATE ;
 
 
-static void usage_exit (void) ;
+static void print_usage (void) ;
 static void interleave_int (STATE * state) ;
 static void interleave_double (STATE * state) ;
 
 
 int
 main (int argc, char **argv)
-{	STATE state ;
+{	STATE *state = NULL ;
 	SF_INFO sfinfo ;
 	int k, double_merge = 0 ;
+	int ret = 1 ;
 
 	if (argc < 5)
 	{	if (argc > 1)
 			puts ("\nError : need at least 2 input files.") ;
-		usage_exit () ;
+		print_usage () ;
+		goto cleanup ;
 		} ;
 
 	if (strcmp (argv [argc - 2], "-o") != 0)
 	{	puts ("\nError : second last command line parameter should be '-o'.\n") ;
-		usage_exit () ;
+		print_usage () ;
+		goto cleanup ;
 		} ;
 
-	if (argc - 2 > MAX_INPUTS)
+	if (argc - 3 > MAX_INPUTS)
 	{	printf ("\nError : Cannot handle more than %d input channels.\n\n", MAX_INPUTS) ;
-		exit (1) ;
+		goto cleanup ;
+		} ;
+
+	state = calloc (1, sizeof (STATE)) ;
+	if (state == NULL)
+	{	puts ("\nError : out of memory.\n") ;
+		goto cleanup ;
 		} ;
 
-	memset (&state, 0, sizeof (state)) ;
 	memset (&sfinfo, 0, sizeof (sfinfo)) ;
 
 	for (k = 1 ; k < argc - 2 ; k++)
 	{
-		if ((state.infile [k - 1] = sf_open (argv [k], SFM_READ, &sfinfo)) == NULL)
+		if ((state->infile [k - 1] = sf_open (argv [k], SFM_READ, &sfinfo)) == NULL)
 		{	printf ("\nError : Not able to open input file '%s'\n%s\n", argv [k], sf_strerror (NULL)) ;
-			exit (1) ;
+			goto cleanup ;
 			} ;
 
 		if (sfinfo.channels != 1)
 		{	printf ("\bError : Input file '%s' should be mono (has %d channels).\n", argv [k], sfinfo.channels) ;
-			exit (1) ;
+			goto cleanup ;
 			} ;
 
 		switch (sfinfo.format & SF_FORMAT_SUBMASK)
@@ -113,28 +121,36 @@ main (int argc, char **argv)
 				break ;
 			} ;
 
-		state.channels ++ ;
+		state->channels ++ ;
 		} ;
 
-	sfinfo.channels = state.channels ;
+	sfinfo.channels = state->channels ;
 	sfinfo.format = sfe_file_type_of_ext (argv [argc - 1], sfinfo.format) ;
 
-	if ((state.outfile = sf_open (argv [argc - 1], SFM_WRITE, &sfinfo)) == NULL)
+	if ((state->outfile = sf_open (argv [argc - 1], SFM_WRITE, &sfinfo)) == NULL)
 	{	printf ("Not able to open output file '%s'\n%s\n", argv [argc - 1], sf_strerror (NULL)) ;
-		exit (1) ;
+		goto cleanup ;
 		} ;
 
 	if (double_merge)
-		interleave_double (&state) ;
+		interleave_double (state) ;
 	else
-		interleave_int (&state) ;
+		interleave_int (state) ;
 
-	for (k = 0 ; k < MAX_INPUTS ; k++)
-		if (state.infile [k] != NULL)
-			sf_close (state.infile [k]) ;
-	sf_close (state.outfile) ;
+	ret = 0 ;
 
-	return 0 ;
+cleanup :
+
+	if (state != NULL)
+	{	for (k = 0 ; k < MAX_INPUTS ; k++)
+			if (state->infile [k] != NULL)
+				sf_close (state->infile [k]) ;
+		sf_close (state->outfile) ;
+		}
+
+	free (state) ;
+
+	return ret ;
 } /* main */
 
 /*------------------------------------------------------------------------------
@@ -142,11 +158,11 @@ main (int argc, char **argv)
 
 
 static void
-usage_exit (void)
+print_usage (void)
 {	puts ("\nUsage : sndfile-interleave <input 1> <input 2> ... -o <output file>\n") ;
 	puts ("Merge two or more mono files into a single multi-channel file.\n") ;
-	exit (0) ;
-} /* usage_exit */
+	printf ("Using %s.\n\n", sf_version_string ()) ;
+} /* print_usage */
 
 
 static void
@@ -158,11 +174,11 @@ interleave_int (STATE * state)
 	{	max_read_len = 0 ;
 
 		for (ch = 0 ; ch < state->channels ; ch ++)
-		{	read_len = sf_read_int (state->infile [ch], state->din.i, BUFFER_LEN) ;
+		{	read_len = (int) sf_read_int (state->infile [ch], state->din.i, BUFFER_LEN) ;
 			if (read_len < BUFFER_LEN)
 				memset (state->din.i + read_len, 0, sizeof (state->din.i [0]) * (BUFFER_LEN - read_len)) ;
 
-			for (k = 0 ; k < read_len ; k++)
+			for (k = 0 ; k < BUFFER_LEN ; k++)
 				state->dout.i [k * state->channels + ch] = state->din.i [k] ;
 
 			max_read_len = MAX (max_read_len, read_len) ;
@@ -184,11 +200,11 @@ interleave_double (STATE * state)
 	{	max_read_len = 0 ;
 
 		for (ch = 0 ; ch < state->channels ; ch ++)
-		{	read_len = sf_read_double (state->infile [ch], state->din.d, BUFFER_LEN) ;
+		{	read_len = (int) sf_read_double (state->infile [ch], state->din.d, BUFFER_LEN) ;
 			if (read_len < BUFFER_LEN)
 				memset (state->din.d + read_len, 0, sizeof (state->din.d [0]) * (BUFFER_LEN - read_len)) ;
 
-			for (k = 0 ; k < read_len ; k++)
+			for (k = 0 ; k < BUFFER_LEN ; k++)
 				state->dout.d [k * state->channels + ch] = state->din.d [k] ;
 
 			max_read_len = MAX (max_read_len, read_len) ;
